@@ -38,7 +38,7 @@
 
 
 /* ----------------------------------------------------------------------------
-   Global definitions, private
+   Global definitions
 */
 
 // stuff for acyclicity checking
